      Program masw2cmp
c Multi-Channel Analysis of Surface Wave generated by an artificial 
c shot and measured along a line
c
c Multi-Channel + Multi-shot version
c
c This program analizes CMP gather files. Use masw2.for to make CMP gather
c files before using this prohram.
c 
c 2.0 20.0 0.001           :fmin,fmax,dt
c CMPCC 1 1                :Method (CMPCC or CMPCH),n_cf_domain,normalize(=0 all, =1 each freq.)
c 25. 200. 1.0             :vmin,vmax,dv
c cmp_13.dat               :File name of CMP gather (A10) for input, should be listed from the shortest spacing
c cmp_13ds.dat             :Output file name (dispersion curve)
c 2.  50  5. 50. 20. 50.   :(f1,v1),(f2,v2),(f3,v3) for lower limit
c 4. 150  5. 100. 20. 100. :(f1,v1),(f2,v2),(f3,v3) for upper limitc
c
c (fmin, fmax)             :Frequency range of analysis
c dt                       :Sampling interval
c CMPCC orCMPCH            :Analysis based on cross-spectra or coherence
c n_cf_domain              :Flag for intermediate outputs for
c                            Stacked seismographs in the velocity-frequency domain
c normalize                :Flag to select the way of normalization
c                            =0 by the maximum over frequency and velocity
c                             (Relative strength of signals is kept)
c                            =1 by the maximum over velocity at each frequency
c                             (Weak signal is enhanced)
c (vmin,vmax,dv)           :Velocity range and interval for analysis
c
c Next two lines are used to limit the range of peak search in C-F domain.
c Next         ==> lower limit of the search range
c Next of next ==> upper limit of the search range
c These can be used to select only one peak for each frequency and reduce the work 
c to do it by hands. 
c
c Input: Common Mid Point gather files  CMP_**.dat
c
c Output: crs_cf.dat	for CMPCC. Graphic output of stacked seismograms in C-F domain
c  	    coh_cf.dat	for CMPCH. Graphic output of stacked seismograms in C-F domain
c         coh_pk.dat	C and F of peaks of stacked seismograms in C-F domain
c     These files are used by GNUPLOT	called by the following scrips.
c              reset
c              unset key
c              # Graph title
c              set title "MASW:Yoshino Park. Jun. 10, 2011"
c              # Horizontal axis: label & range
c              set xlabel "Frequency (Hz)"
c              set xrange [0:20]
c              # Verical axis: label & range
c              set ylabel "Phase Velocity (m/s)"
c              set yrange [0:200]
c              # C-F spectra 
c              set zrange [0:1]
c              #
c              set pm3d map
c              set multiplot
c              # arrange the paths
c              splot "c://exploration/masw/maswkf/crs_cf.dat"
c              splot "c://exploration/masw/maswkf/coh_pk.dat" with points pt 1 lt 8
c              unset multiplot
c 		cmp_**ds.dat Output file uncluding dispersion curve
c
      parameter(nn0=4096,nch0=24,nvl0=2000,lmax0=500)
      complex ci,cf(nn0),crr(nn0,nch0),csum
      real aw(nn0,nvl0),awmax,cr(nn0,nch0)
      real  f(nn0),g(nn0),g1(nn0*2),g2(nn0*2),w(lmax0)
      real  fr(nn0),rr(nch0),
     *      y(nvl0),dy(nvl0),x_vel(100),aw_vel(100)
	character filen*80,finput*12,cmethod*5,cform*10
       character cdrv*2,cwkf*9,cnum*3,com1*80
       data cdrv/'./'/ 
c     character cdrv*3,cwkf*10  
c     data cdrv/'../'/  
      cmethod='CMPCC'
      data pai/3.1415926/ c0,ci/(0.,0.),(0.,1.)/ eps/1.e-6/
      write(6,*)'Program masw2cmp.for'
c working folder name
	cwkf=cdrv//'maswkf/'
c parameter file
      filen='./maswkf/prm/'//'masw2cmp.prm'
      write(6,*)filen
      open(11,file=filen,status='old')
c frequency range for analysis
      read(11,*)fmin,fmax,dt,n_par
      write(6,*)fmin,fmax,dt,n_par
      read(11,*)n_cf_domain,normalize
      write(6,*)n_cf_domain,normalize
c file name for field data
      read(11,*)vmin,vmax,dv
      write(6,*)vmin,vmax,dv
      nvl=int((vmax-vmin)/dv+1)
      if(nvl.gt.nvl0) then
       write(6,*)' nvl.gt.nvl0'
       stop
      endif
c read CMP gather files
c     read(11,*)finput
      read(11,*)ncmps,ncmpe,ncmpd
      read(11,*)ff1,fv1,ff2,fv2,ff3,fv3		   ! lower limit
      read(11,*)gg1,gv1,gg2,gv2,gg3,gv3		   ! upper limit
c    
      filen='./maswkf/prm/gnuplt_script/'//'multi_cf.plt'
      open(8,file=filen,status='unknown')
c  
      do icmp=ncmps,ncmpe,ncmpd
c
      if(icmp.lt.10) then
       write(cnum,'(a2,i1)')'00',icmp
      else if(icmp.lt.100) then
       write(cnum,'(a1,i2)')'0',icmp
      else if(icmp.lt.1000) then
       write(cnum,'(i3)')icmp
      endif
c
      finput='cmp'//cnum//'.dat' 
      filen='./maswkf/data/cmp_gathers/'//finput
      write(6,*)filen,' (CMP gather) '
      open(13,file=filen,status='unknown')
      read(13,'(i8,2f8.4,i8)')icmp_,x_cmp,dt,nn
      write(6,'(i8,2f8.4,i8)')icmp_,x_cmp,dt,nn
      do icase=1,nch0						! icase=i_spacing
       read(13,'(i8,4f8.3)',end=100)irr_,rr(icase),x1,x_cmp,x2
       write(6,'(i8,4f8.3)')irr_,rr(icase),x1,x_cmp,x2
       read(13,'(5e13.6)')(cr(i,icase),i=1,nn)
      enddo
  100 ncase=icase-1
      close(13)    
c
      ndur=nn0/nn
      nn=nn*ndur
c duration & frequency
	td=real(nn)*dt
      do i=1,nn
       fr(i)=real(i-1)/td
      enddo
c     isf=int(fmin*td)+1
      isf=int(min(fmin,2.0)*td)+1
c     ief=int(fmax*td)
      ief=int(max(fmax,20.)*td)
c
      dx=rr(ncase)/real(ncase)
      dr=1.*dx
c
      do icase=1,ncase   
       do i=1,nn
        cf(i)=cr(i,icase)    
       enddo
       call cfft(cf,nn,-1)
c Parzen window
        if(n_par.ne.0) then
         bw=3.8/td
        endif
        do i=1,nn
         f(i)= real(cf(i))
         g(i)=aimag(cf(i))
        enddo
        if(bw.gt.1.e-6) then
         call w_parzen(f,nn,nn0,dt,bw,g1,g2,nn0*2,w,lmax0)
         call w_parzen(g,nn,nn0,dt,bw,g1,g2,nn0*2,w,lmax0)
        endif
        do i=1,nn
         cf(i)=cmplx(f(i),g(i))
        enddo
c
       do i=1,nn
        crr(i,icase)=cf(i)*td/real(nn)
       enddo
      enddo
c
      if(cmethod.eq.'CMPCH') then
       do icase=1,ncase
        do i=1,nn
         crr(i,icase)=crr(i,icase)/cabs(crr(i,icase))
        enddo
       enddo
      endif
c conversion to c-f domain from r-f domain by stacking with phase lag correction.
	do i=1,nn0
	 do ivl=1,nvl0
	  aw(i,ivl)=0.0
	 enddo
	enddo
      do i=isf,ief
	 do ivl=1,nvl
        vl=vmin+(ivl-1)*dv
	  rmd=vl/fr(i)
        ak=2.*pai/rmd
        csum=c0
	  nsum=0
        do icase=1,ncase
         csum=csum+crr(i,icase)*exp(ci*ak*rr(icase))
         nsum=nsum+1
	  enddo
	  aw(i,ivl)=real(csum)
	 enddo
	enddo
      awmax=aw(i,1)
      do i=isf,ief
       do ivl=2,nvl
        if(awmax.lt.aw(i,ivl)) awmax=aw(i,ivl)
       enddo
      enddo
      do i=isf,ief
       do ivl=1,nvl
        aw(i,ivl)=aw(i,ivl)/awmax
        if(aw(i,ivl).lt.0.) aw(i,ivl)=0.
       enddo
      enddo
c
      if(normalize.eq.1) then
       do i=isf,ief
        awmax=aw(i,1)
        do ivl=2,nvl
         if(awmax.lt.aw(i,ivl)) awmax=aw(i,ivl)
        enddo
        do ivl=1,nvl
         if(awmax.gt.eps) then
	    aw(i,ivl)=aw(i,ivl)/awmax
         else
          aw(i,ivl)=0.
         endif
        enddo
       enddo
	endif
c intermediate output: c-f domain
      if(n_cf_domain.ne.0) then
       if(cmethod.eq.'CMPCC') then
	  filen='./maswkf/data/c_f_panels/'//'crs_cf'//cnum//'.dat'
       else if(cmethod.eq.'CMPCH') then
	  filen='./maswkf/data/c_f_panels/'//'coh_cf'//cnum//'.dat'
       endif
       write(6,*)filen
	 open(2,file=filen,status='unknown')
       filen='./maswkf/data/c_f_panels/'//'coh_pk'//cnum//'.dat'
       write(6,*)'Interim outout (peaks of C-F spectra):',filen
	 open(22,file=filen,status='unknown')
       write(2,'(a1,7x,a8,2(8x,a8))')
     *        '#',' Freq.  ','  Vel.  ',' C-F Sp.'
       write(22,'(a1,7x,a8,2(8x,a8))')
     *        '#',' Freq.  ','  Vel.  ',' C-F Sp.'
       do i=isf,ief
	  do ivl=1,nvl
         write(2,*)fr(i),vmin+(ivl-1)*dv,aw(i,ivl)
        enddo
        write(2,*)
	 enddo
       close(2)
      endif
c output file
        finput='cmp'//cnum//'ds.dat'
	filen='./maswkf/data/dispersion/'//finput
      write(6,*)filen
      open(12,file=filen,status='unknown')
      write(12,'(2(1x,a8))')
     *        '#  Freq.',' V.m/s  '
c determination of phase velocity
      isf=int(fmin*td)+1
      ief=int(fmax*td)
      do i=isf,ief
       if(fr(i).lt.ff1) then
        fvmin=fv1
       else if(ff1.le.fr(i).and.fr(i).lt.ff2) then
        fvmin=(fv2-fv1)*(fr(i)-ff1)/(ff2-ff1)+fv1
       else if(ff2.le.fr(i).and.fr(i).lt.ff3) then
        fvmin=(fv3-fv2)*(fr(i)-ff2)/(ff3-ff2)+fv2
       else if(ff3.le.fr(i)) then
        fvmin=fv3
       endif
       if(fr(i).lt.gg1) then
        fvmax=gv1
       else if(gg1.le.fr(i).and.fr(i).lt.gg2) then
        fvmax=(gv2-gv1)*(fr(i)-gg1)/(gg2-gg1)+gv1
       else if(gg2.le.fr(i).and.fr(i).lt.gg3) then
        fvmax=(gv3-gv2)*(fr(i)-gg2)/(gg3-gg2)+gv2
       else if(gg3.le.fr(i)) then
        fvmax=gv3
       endif
       do ivl=1,nvl
        y(ivl)=aw(i,ivl)
       enddo
c differenciation over velocity
       do ivl=1,nvl-1
        dy(ivl)=(y(ivl+1)-y(ivl))/dv
       enddo
c search local maximum
       n_max=0
       do ivl=2,nvl
        if(dy(ivl).lt.0..and.dy(ivl-1).gt.0.) then
         ramda=(vmin+real(ivl-1)*dv)/fr(i)
         if(ramda.ge.2.*dr.and.ramda.le.4.*rr(ncase)) then
          xvel=vmin+real(ivl-1)*dv
          if(xvel.gt.fvmin.and.xvel.lt.fvmax) then
           n_max=n_max+1
           x_vel(n_max)=xvel
           aw_vel(n_max)=y(ivl)
           write(22,*)fr(i),x_vel(n_max),y(ivl)
          endif
         endif
        endif
       enddo
       if(n_max.le.8) then
        write(cform,'(a1,i1,a5)') '(',n_max+1,'f9.3)'
        if(n_max.gt.0) then 
         write(12,cform(1:7) )fr(i),(x_vel(i_max),i_max=1,n_max)
        endif
       else if(n_max.le.98) then
        write(cform,'(a1,i2,a5)') '(',n_max+1,'f9.3)'
        write(12,cform(1:8) )fr(i),(x_vel(i_max),i_max=1,n_max)
       endif
      enddo
      close(12)
      close(11)
c graphic output using GNUPLOT
      filen='./maswkf/prm/geometry.prm'
      open(11,file=filen,status='old')
      write(6,*)filen
      read(11,'(a45)')com1
      write(6,'(a45)')com1
      close(11)
      filen='./maswkf/prm/gnuplt_script/c_f_panels/masw'//cnum//'.plt'
      write(6,*)filen
      open(12,file=filen,status='unknown')
      write(12,*)'reset'
      write(12,*)'unset key'
      write(12,*)'# Graph title'
      write(12,*)'set title "'//com1(1:45)//': CMP#'//cnum//' "'
      write(12,*)'# Horizontal axis: label & range'
      write(12,*)'set xlabel "Frequency (Hz)"'
      write(12,*)'set xrange [0:20]'
      write(12,*)'# Verical axis: label & range'
      write(12,*)'set ylabel "Phase Velocity (m/s)"'
      write(12,*)'set yrange [0:400]'
      write(12,*)'set grid'
      write(12,*)'# C-F spectra '
      write(12,*)'set zrange [0:1]'
      write(12,*)'#'
      write(12,*)'set pm3d map'
      write(12,*)'set multiplot'
      write(12,*)'splot "./maswkf/data/c_f_panels/crs_cf'//cnum//'.dat"'
      write(12,*)'splot "./maswkf/data/c_f_panels/coh_pk'//cnum//'.dat"'
     *                                    //' with points pt 1 lt 8'
      write(12,*)'unset multiplot'
      write(12,*)'set terminal postscript color enhanced'
      write(12,*)
     *     'set output "./maswkf/data/c_f_panels/fig/cmp'//cnum//'.ps"'
      write(12,*)'# Graph title'
      write(12,*)'set title "'//com1(1:45)//': CMP#'//cnum//' "'
      write(12,*)'# Horizontal axis: label & range'
      write(12,*)'set xlabel "Frequency (Hz)"'
      write(12,*)'set xrange [0:20]'
      write(12,*)'# Verical axis: label & range'
      write(12,*)'set ylabel "Phase Velocity (m/s)"'
      write(12,*)'set yrange [0:400]'
      write(12,*)'set grid'
      write(12,*)'# C-F spectra '
      write(12,*)'set zrange [0:1]'
      write(12,*)'#'
      write(12,*)'set pm3d map'
      write(12,*)'set multiplot'
      write(12,*)'splot "./maswkf/data/c_f_panels/crs_cf'//cnum//'.dat"'
      write(12,*)'splot "./maswkf/data/c_f_panels/coh_pk'//cnum//'.dat"'
     *                                    //' with points pt 1 lt 8'
      write(12,*)'unset multiplot'
      write(12,*)'set output'
      write(12,*)'set terminal wxt'
      close(12)
c
      write(8,*)
     * 'load ''./maswkf/prm/gnuplt_script/c_f_panels/masw',cnum,'.plt'''
c
      if(icmp+ncmpd.lt.10) then
       write(cnum,'(a2,i1)')'00',icmp+ncmpd
      else if(icmp+ncmpd.lt.100) then
       write(cnum,'(a1,i2)')'0',icmp+ncmpd
      else if(icmp+ncmpd.lt.1000) then
       write(cnum,'(i3)')icmp+ncmpd
      endif
      write(8,*)'pause -1 "Hit return to continue to cmp',cnum,'"'
c
      enddo
      close(8)
c
      stop
      end
c+++++
      subroutine peak_rms(x,n,judge,ac)
      real x(n)
c peak value
c      write(6,*)'peak_rms'
	amx=abs(x(1))
	do i=2,n
	 if(amx.lt.abs(x(i))) amx=abs(x(i))
	enddo
c rms value
      sum=0.0
      do i=1,n
       sum=sum+x(i)**2
	enddo
      rms=sqrt(sum/real(n))
c judge
      judge=1
      if(amx.gt.rms*ac) judge=0
	return
	end
c+++++
      subroutine taper(x,n,dt)
      real x(n)
      data pai/3.141592/
      mwin=int(real(n)/20.+0.5)
c linear trend fitting
      b1=x(1)
      do i=2,n
       b1=b1+real(i)*x(i)
      enddo
      b2=x(1)
      do i=1,n
       b2=b2+x(i)
      enddo
      a0=real(n*(n+1)*(2*n+1))/6.*dt
      b0=real(n*(n+1))/2.
      c0=b0*dt
      d0=real(n)
      f0=a0*d0-b0*c0
      aa=( d0*b1-b0*b2)/f0
      bb=(-c0*b1+a0*b2)/f0
      do i=1,n
       x(i)=x(i)-aa*real(i)*dt-bb
      enddo
c tapering
      do i=1,mwin
       x(i)=x(i)*SIN(PAI/2.*REAL(i-1)/REAL(MWIN))
      enddo
      do i=n-mwin,n
       x(i)=x(i)*SIN(PAI/2.*REAL(n-i)/REAL(MWIN))
      enddo
      return
      end
c+++++
      SUBROUTINE CFFT(X,N,IND)
c...........................................................................
c
c     Complex Fast Fourier Transform
c
c     Coded by Y. OHSAKI in "Introduction to spectral analysis for
c                            seismic waves"
c     Typed by D.Suetsugu, IISEE
c     Modified by H.Inoue, IISEE
c
c     Usage.......   CALL FFT(N,X,IND)
c
c     where
c
c     X(N)    In/Out  Equi-spaced N complex data/Transformed values
c     N       In      Number of data, MUST be a power of 2.
c     IND     In      -1=Fourier transform,  1=Inverse Fourier transform
c
c     Note..... You must divide the output X() by N to get Fourier
c               coefficient when IND=-1.
c
c...........................................................................
      complex x(n),temp,theta
c
      j=1
      do 140 i=1,n
        if(i.ge.j) goto 110
        temp=x(j)
        x(j)=x(i)
        x(i)=temp
  110   m=n/2
  120   if(j.le.m) goto 130
        j=j-m
        m=m/2
        if(m.ge.2) goto 120
  130   j=j+m
  140 continue
c  
      kmax=1
      do 150 kkk=1,9999
        if(kmax.ge.n) return
        istep=kmax*2
        do 170 k=1,kmax
          theta=cmplx(0.0,3.141593*float(ind*(k-1))/float(kmax))
          do 160 i=k,n,istep
            j=i+kmax
            temp=x(j)*cexp(theta)
            x(j)=x(i)-temp
            x(i)=x(i)+temp
  160     continue
  170   continue
        kmax=istep
  150 continue
      end
c+++++
      subroutine w_parzen(h,nn,n,dt,bw,g1,g2,n2,w,lmax0)
      real h(n),g1(n2),g2(n2),w(lmax0)
      data pai/3.141592/
c parzen window
      df=1./(dt*real(nn))
      udf=1.854305/bw*df
      if(udf.gt.0.5) then
       write(6,*)' Bandwidth is too narrow'
      endif
      lmax=int(2./udf+0.5)
      if(lmax.gt.lmax0) then
       write(6,*)'Bandwidth is too wide'
      endif
      w(1)=0.75*udf
      do l=2,lmax
       dif=pai/2.*real(l-1)*udf
       w(l)=w(1)*(sin(dif)/dif)**4
      enddo
      ll=lmax*2-1
      ln=ll-1+nn
      lt=(ll-1)*2+nn
      le=lt-lmax+1
      do k=1,lt
       g1(k)=0.
      enddo
      do k=1,nn
       g1(ll-1+k)=h(k)
      enddo
      do k=lmax,le
       s=w(1)*g1(k)
       do l=2,lmax
        s=s+w(l)*(g1(k-l+1)+g1(k+l-1))
       enddo
       g2(k)=s
      enddo
      do l=2,lmax
       g2(ll+l-1)=g2(ll+l-1)+g2(ll-l+1)
       g2(ln-l+1)=g2(ln-l+1)+g2(ln+l-1)
      enddo
      do k=1,nn
       h(k)=g2(ll-1+k)
      enddo
      return
      end

